//+------------------------------------------------------------------+
//|                                               DB_Strategy_v2.mq4 |
//|                                Copyright  2010, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of MAs:
// MA_Method= 0: SMA        - Simple Moving Average
// MA_Method= 1: EMA        - Exponential Moving Average
// MA_Method= 2: Wilder     - Wilder Exponential Moving Average
// MA_Method= 3: LWMA       - Linear Weighted Moving Average 
// MA_Method= 4: SineWMA    - Sine Weighted Moving Average
// MA_Method= 5: TriMA      - Triangular Moving Average
// MA_Method= 6: LSMA       - Least Square Moving Average (or EPMA, Linear Regression Line)
// MA_Method= 7: SMMA       - Smoothed Moving Average
// MA_Method= 8: HMA        - Hull Moving Average by Alan Hull
// MA_Method= 9: ZeroLagEMA - Zero-Lag Exponential Moving Average
// MA_Method=10: DEMA       - Double Exponential Moving Average by Patrick Mulloy
// MA_Method=11: T3         - T3 by T.Tillson
// MA_Method=12: ITrend     - Instantaneous Trendline by J.Ehlers
// MA_Method=13: Median     - Moving Median
// MA_Method=14: GeoMean    - Geometric Mean
// MA_Method=15: REMA       - Regularized EMA by Chris Satchwell
// MA_Method=16: ILRS       - Integral of Linear Regression Slope 
// MA_Method=17: IE/2       - Combination of LSMA and ILRS 
// MA_Method=18: TriMAgen   - Triangular Moving Average generalized by J.Ehlers
// MA_Method=19: VWMA       - Volume Weighted Moving Average 
// MA_Method=20: JSmooth    - Smoothing by Mark Jurik
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4

 
#property copyright "Copyright  2009-10, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 8
#property indicator_color1 LightBlue
#property indicator_color2 Tomato
#property indicator_width1 2
#property indicator_width2 2
#property indicator_color3 LightBlue
#property indicator_color4 Tomato
#property indicator_width3 2
#property indicator_width4 2
#property indicator_color5 Blue
#property indicator_color6 Red
#property indicator_width5 1
#property indicator_width6 1
#property indicator_color7 Blue
#property indicator_color8 Red
#property indicator_width7 1
#property indicator_width8 1


//---- 

int     TimeFrame         =  0;       //TimeFrame in min
extern int     Price             =  0;       //Price = 0...6 (see List of Prices)
extern int     HiMALength        = 15;       //High MA Period
extern int     LoMALength        = 15;       //Low  MA Period
extern int     HiMAMode          =  0;       //High MA Method = 0...20 (see List of MAs) 
extern int     LoMAMode          =  0;       //Low  MA Mathod = 0...20 (see List of MAs) 
extern int     PivotLength       =  4;       //
extern int     SignalMode        =  1;       //Switch of Signal mode(0-off,1-on)
extern int     PivotSignals      =  1;       //0-off,1-on
extern int     ReversalSignals   =  1;       //0-off,1-on
extern int     AlertMode         =  1;       //Sound Alert switch(0...2) 
extern int     WarningMode       =  0;       //Warning Mode (0-off,1-on)

double UpPivot[];
double LoPivot[];
double LongBk[];
double ShortBk[];
double LongEx[];
double ShortEx[]; 
double LongF[];
double ShortF[];

//----
double tmp[][12];
int    draw_begin, mBars, pBars, mcnt_bars; 
string up_name, lo_name;
datetime pTime,prevTime;
bool   UpTrendAlert=false, DnTrendAlert=false;
string TF, setup; 
double hhArr[], llArr[],bufArr[];
string prevmess ="";
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- 
      SetIndexStyle(0,DRAW_LINE);
      SetIndexStyle(1,DRAW_LINE);
     
      
      if(SignalMode > 0)
      {
      SetIndexStyle(2,DRAW_ARROW);
      SetIndexArrow(2,233);
      
      SetIndexStyle(3,DRAW_ARROW);
      SetIndexArrow(3,234);
      
      SetIndexStyle(4,DRAW_ARROW);
      SetIndexArrow(4,233);
      
      SetIndexStyle(5,DRAW_ARROW);
      SetIndexArrow(5,234);
      
      SetIndexStyle(6,DRAW_ARROW);
      SetIndexArrow(6,233);
      
      SetIndexStyle(7,DRAW_ARROW);
      SetIndexArrow(7,234);
      }
      
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   
   if(TimeFrame == 0 || TimeFrame < Period()) TimeFrame = Period();
    
   draw_begin=MathMax(HiMALength,LoMALength)*TimeFrame/Period();
//---- 
   switch(HiMAMode)
   {
   case 1 : up_name="EMA(";  break;
   case 2 : up_name="Wilder("; break;
   case 3 : up_name="LWMA("; break;
   case 4 : up_name="SineWMA("; break;
   case 5 : up_name="TriMA("; break;
   case 6 : up_name="LSMA("; break;
   case 7 : up_name="SMMA("; break;
   case 8 : up_name="HMA("; break;
   case 9 : up_name="ZeroLagEMA("; break;
   case 10: up_name="DEMA(";  break;
   case 11: up_name="T3(";  break;
   case 12: up_name="InstTrend(";  break;
   case 13: up_name="Median(";  break;
   case 14: up_name="GeometricMean("; break;
   case 15: up_name="REMA(";  break;
   case 16: up_name="ILRS(";  break;
   case 17: up_name="IE/2(";  break;
   case 18: up_name="TriMA_gen("; break;
   case 19: up_name="VWMA("; break;
   case 20: up_name="JSmooth("; break;
   default: HiMAMode=0; up_name="SMA(";
   }
   
   switch(LoMAMode)
   {
   case 1 : lo_name="EMA(";  break;
   case 2 : lo_name="Wilder("; break;
   case 3 : lo_name="LWMA("; break;
   case 4 : lo_name="SineWMA("; break;
   case 5 : lo_name="TriMA("; break;
   case 6 : lo_name="LSMA("; break;
   case 7 : lo_name="SMMA("; break;
   case 8 : lo_name="HMA("; break;
   case 9 : lo_name="ZeroLagEMA("; break;
   case 10: lo_name="DEMA(";  break;
   case 11: lo_name="T3(";  break;
   case 12: lo_name="InstTrend(";  break;
   case 13: lo_name="Median(";  break;
   case 14: lo_name="GeometricMean("; break;
   case 15: lo_name="REMA(";  break;
   case 16: lo_name="ILRS(";  break;
   case 17: lo_name="IE/2(";  break;
   case 18: lo_name="TriMA_gen("; break;
   case 19: lo_name="VWMA("; break;
   case 20: lo_name="JSmooth("; break;
   default: LoMAMode=0; lo_name="SMA(";
   }
   
   switch(TimeFrame)
   {
   case 1     : TF = "M1"; break;
   case 5     : TF = "M5"; break;
   case 15    : TF = "M15"; break;
   case 30    : TF = "M30"; break;
   case 60    : TF = "H1"; break;
   case 240   : TF = "H4"; break;
   case 1440  : TF = "D1"; break;
   case 10080 : TF = "W1"; break;
   case 43200 : TF = "MN1"; break;
   default    : TF = "Current";
   } 
      
   SetIndexDrawBegin(0,draw_begin);
   SetIndexLabel(0,"Upper["+TF+"]("+up_name+HiMALength+"))");
   SetIndexLabel(1,"Lower["+TF+"]("+lo_name+LoMALength+"))");
      /*if(SignalMode > 0)
      {
      SetIndexLabel(2,"ShortSignal["+TF+"]("+LoBandPrice+","+up_name+LoBandPeriod+","+DoubleToStr(LoBandRisk,4)+")");
      SetIndexLabel(3,"ShortSignal["+TF+"]("+UpBandPrice+","+lo_name+UpBandPeriod+","+DoubleToStr(UpBandRisk,4)+")");
      }*/
//---- 
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
//----      
   IndicatorBuffers(8);
   SetIndexBuffer(0,UpPivot);
   SetIndexBuffer(1,LoPivot);
         
      if(SignalMode > 0)
      {
      SetIndexBuffer(2,LongBk);
      SetIndexBuffer(3,ShortBk);
      SetIndexBuffer(4,LongEx);
      SetIndexBuffer(5,ShortEx);
      SetIndexBuffer(6,LongF);
      SetIndexBuffer(7,ShortF);
      
      SetIndexDrawBegin(2,draw_begin);
      SetIndexDrawBegin(3,draw_begin);
      SetIndexDrawBegin(4,draw_begin);
      SetIndexDrawBegin(5,draw_begin);
      SetIndexDrawBegin(6,draw_begin);
      SetIndexDrawBegin(7,draw_begin);
      }
   
//----   
   setup = TF + Price + HiMALength + LoMALength + HiMAMode + LoMAMode + PivotLength + SignalMode; // + AlertMode + WarningMode       =  0;

   ArrayResize(hhArr,PivotLength);
   ArrayResize(llArr,PivotLength);
   ArrayResize(bufArr,PivotLength);
   
   return(0);
}

int deinit() 
{
   Comment("");
   ObjectDel();
}

//+------------------------------------------------------------------+
//| DB_Strategy_v1                                                   |
//+------------------------------------------------------------------+
int start()
{
   int limit, y, i, shift, cnt_bars=IndicatorCounted(); 
   double aPrice[], upPrice[], loPrice[], mUpMA[], mLoMA[], mUp[], mDn[], mHi[], mLo[], trend[];
   double mLongF[], mShortF[], bk[], ex[], mLongBk[], mShortBk[], mLongEx[], mShortEx[], fail[], failsig[],dir[];
   int   LastShortBar[], LastLongBar[];
   
   double _Point = Point * MathPow(10,Digits%2);
   
   if(TimeFrame!=Period()) mBars = iBars(NULL,TimeFrame); else mBars = Bars;   
   
   if(mBars != pBars)
   {
   ArrayResize(aPrice,mBars);
   ArrayResize(upPrice,mBars);
   ArrayResize(loPrice,mBars);
   ArrayResize(mUpMA,mBars);
   ArrayResize(mLoMA,mBars);   
   ArrayResize(mUp,mBars);
   ArrayResize(mDn,mBars);
   ArrayResize(mHi,mBars);
   ArrayResize(mLo,mBars);
   ArrayResize(trend,mBars);
   
      if(SignalMode > 0)
      {
      ArrayResize(mLongF,mBars);
      ArrayResize(mShortF,mBars);
      ArrayResize(mLongBk,mBars);
      ArrayResize(mShortBk,mBars);
      ArrayResize(mLongEx,mBars);
      ArrayResize(mShortEx,mBars);
      ArrayResize(bk,mBars);
      ArrayResize(ex,mBars);
      ArrayResize(fail,mBars);
      ArrayResize(failsig,mBars);
      ArrayResize(dir,mBars);
      ArrayResize(LastShortBar,mBars);
      ArrayResize(LastLongBar,mBars);
      }
   
   if((HiMAMode==10 || HiMAMode==11 || HiMAMode==20) 
   || (LoMAMode==10 || LoMAMode==11 || LoMAMode==20)) ArrayResize(tmp,mBars);
      
      
   pBars = mBars;
   }  
   
   if(cnt_bars<1)
   {
      for(i=Bars-1;i>0;i--) 
      { 
      UpPivot[i] = EMPTY_VALUE; 
      LoPivot[i] = EMPTY_VALUE; 
      LongF[i]= EMPTY_VALUE;
      ShortF[i]= EMPTY_VALUE;
      LongBk[i]= EMPTY_VALUE;
      ShortBk[i]= EMPTY_VALUE;
      LongEx[i]= EMPTY_VALUE;
      ShortEx[i]= EMPTY_VALUE;
      }
   mcnt_bars = 0;
   
   }
//---- 
   if(mcnt_bars > 0) mcnt_bars--;
   
   for(y=mcnt_bars;y<mBars;y++)
   {
      aPrice[y] = iMA(NULL,TimeFrame,1,0,0,Price,mBars-y-1);   
                  
      upPrice[y] = iMA(NULL,TimeFrame,1,0,0,PRICE_HIGH,mBars-y-1);
      loPrice[y] = iMA(NULL,TimeFrame,1,0,0,PRICE_LOW ,mBars-y-1); 
      
      double mlo  = iLow (NULL,TimeFrame,mBars-y-1); 
      double mhi  = iHigh(NULL,TimeFrame,mBars-y-1); 
      double mlo1 = iLow (NULL,TimeFrame,mBars-y); 
      double mhi1 = iHigh(NULL,TimeFrame,mBars-y); 
      
      switch(HiMAMode)
      {
      case 1 : mUpMA[y] = EMA(upPrice[y],mUpMA,HiMALength,y); break;
      case 2 : mUpMA[y] = Wilder(upPrice,mUpMA,HiMALength,y); break;  
      case 3 : mUpMA[y] = LWMA(upPrice,HiMALength,y); break;
      case 4 : mUpMA[y] = SineWMA(upPrice,HiMALength,y); break;
      case 5 : mUpMA[y] = TriMA(upPrice,HiMALength,y); break;
      case 6 : mUpMA[y] = LSMA(upPrice,HiMALength,y); break;
      case 7 : mUpMA[y] = SMMA(upPrice,mUpMA,HiMALength,y); break;
      case 8 : mUpMA[y] = HMA(upPrice,HiMALength,y); break;
      case 9 : mUpMA[y] = ZeroLagEMA(upPrice,mUpMA,HiMALength,y); break;
      case 10: mUpMA[y] = DEMA(0,upPrice[y],HiMALength,1,y); break;
      case 11: mUpMA[y] = T3(0,upPrice[y],HiMALength,0.7,y); break;
      case 12: mUpMA[y] = ITrend(upPrice,mUpMA,HiMALength,y); break;
      case 13: mUpMA[y] = Median(upPrice,HiMALength,y); break;
      case 14: mUpMA[y] = GeoMean(upPrice,HiMALength,y); break;
      case 15: mUpMA[y] = REMA(upPrice[y],mUpMA,HiMALength,0.5,y); break;
      case 16: mUpMA[y] = ILRS(upPrice,HiMALength,y); break;
      case 17: mUpMA[y] = IE2(upPrice,HiMALength,y); break;
      case 18: mUpMA[y] = TriMA_gen(upPrice,HiMALength,y); break;
      case 19: mUpMA[y] = VWMA(upPrice,HiMALength,y); break;
      case 20: mUpMA[y] = JSmooth(0,upPrice[y],HiMALength,1,y); break;
      default: mUpMA[y] = SMA(upPrice,HiMALength,y); break;
      }
   
      switch(LoMAMode)
      {
      case 1 : mLoMA[y] = EMA(loPrice[y],mLoMA,LoMALength,y); break;
      case 2 : mLoMA[y] = Wilder(loPrice,mLoMA,LoMALength,y); break;  
      case 3 : mLoMA[y] = LWMA(loPrice,LoMALength,y); break;
      case 4 : mLoMA[y] = SineWMA(loPrice,LoMALength,y); break;
      case 5 : mLoMA[y] = TriMA(loPrice,LoMALength,y); break;
      case 6 : mLoMA[y] = LSMA(loPrice,LoMALength,y); break;
      case 7 : mLoMA[y] = SMMA(loPrice,mLoMA,LoMALength,y); break;
      case 8 : mLoMA[y] = HMA(loPrice,LoMALength,y); break;
      case 9 : mLoMA[y] = ZeroLagEMA(loPrice,mLoMA,LoMALength,y); break;
      case 10: mLoMA[y] = DEMA(6,loPrice[y],LoMALength,1,y); break;
      case 11: mLoMA[y] = T3(6,loPrice[y],LoMALength,0.7,y); break;
      case 12: mLoMA[y] = ITrend(loPrice,mLoMA,LoMALength,y); break;
      case 13: mLoMA[y] = Median(loPrice,LoMALength,y); break;
      case 14: mLoMA[y] = GeoMean(loPrice,LoMALength,y); break;
      case 15: mLoMA[y] = REMA(loPrice[y],mLoMA,LoMALength,0.5,y); break;
      case 16: mLoMA[y] = ILRS(loPrice,LoMALength,y); break;
      case 17: mLoMA[y] = IE2(loPrice,LoMALength,y); break;
      case 18: mLoMA[y] = TriMA_gen(loPrice,LoMALength,y); break;
      case 19: mLoMA[y] = VWMA(loPrice,LoMALength,y); break;
      case 20: mLoMA[y] = JSmooth(6,loPrice[y],LoMALength,1,y); break;
      default: mLoMA[y] = SMA(loPrice,LoMALength,y); break;
      }
            
      trend[y] = trend[y-1];
      mDn[y]   = mDn[y-1];
      mUp[y]   = mUp[y-1]; 
      mLo[y]   = mLo[y-1];
      mHi[y]   = mHi[y-1];   
      
      
      if(aPrice[y] > mUpMA[y] && trend[y-1]<=0) trend[y]= 1; 
      if(aPrice[y] < mLoMA[y] && trend[y-1]>=0) trend[y]=-1;  
 
      if(PivotSignals > 0) 
      {
      string a_name = "Pivots" + setup + "_" + TimeToStr(Time[mBars-y-1]);
      ObjectDelete(a_name); 
      double diff = _Point;
      }
            
      if(trend[y] > 0)
      {  
      mHi[y] = MathMax(mhi,mHi[y-1]);
         
         if(trend[y-1] <= 0)
         { 
         mLo[y]    = 100000;
         mDn[y] = MathMin(mLo[y-1],mlo);
            if(PivotSignals > 0 && mDn[y] > mDn[y-1])
            {
            drawArrow("Pivots",Time[mBars-y-1],NormalizeDouble(Low[mBars-y-1],Digits),"Wingdings 3",11,"\xC7",Lime); //mLongSig[y] = mlo;
            if(WarningMode > 0 && y==mBars-1) PlaySound("alert2.wav");
            }
         }
      }
      
      if (trend[y] < 0) 
      {
      mLo[y] = MathMin(mlo,mLo[y-1]);   
         
         if(trend[y-1] >= 0) 
         {
         mHi[y]    =-100000;
         mUp[y] = MathMax(mhi,mHi[y-1]);
            if(PivotSignals > 0 && mUp[y] < mUp[y-1])
            { 
            drawArrow("Pivots",Time[mBars-y-1],NormalizeDouble(High[mBars-y-1] + 3*diff,Digits),"Wingdings 3",11,"\xC8",Magenta); //mShortSig[y] = mhi;
            if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav");  
            }
         }
      }
      
     
      if(trend[y-1] != trend[y-2] && iTime(NULL,TimeFrame,mBars-y-1) != prevTime)//isNewBar(mBars-y-1))
      {   
      for(i = PivotLength - 1; i > 0; i--) hhArr[i] = hhArr[i - 1];
      
      if(trend[y-1] > 0) hhArr[0] = mDn[y-2];
      if(trend[y-1] < 0) hhArr[0] = mUp[y-2];
      prevTime = iTime(NULL,TimeFrame,mBars-y-1);
      }
      
      /*
      Comment("P0=" + DoubleToStr(hhArr[0],Digits) + "\n" 
            + "P1=" + DoubleToStr(hhArr[1],Digits) + "\n" 
            + "P2=" + DoubleToStr(hhArr[2],Digits) + "\n" 
            + "P3=" + DoubleToStr(hhArr[3],Digits) + "\n"
            + "pT=" + TimeToStr(prevTime)); 
      */
      
      if(SignalMode > 0)
      {
      bk[y] = bk[y-1];
      fail[y] = fail[y-1];
      failsig[y] = failsig[y-1];
      dir[y] = dir[y-1];
      LastShortBar[y] = LastShortBar[y-1];
      LastLongBar[y]  = LastLongBar[y-1];
      
      if(mlo > mUp[y] && bk[y-1]<=0) bk[y]= 1;     
      if(mhi < mDn[y] && bk[y-1]>=0) bk[y]=-1;
     
      mLongBk[y] = EMPTY_VALUE;
      mShortBk[y] = EMPTY_VALUE;   
         
         if(bk[y] > 0)
         {
            if(bk[y-1] <= 0) 
            {
            mLongBk[y] = mlo;
            failsig[y] = 0;
            dir[y] = 1;
            LastLongBar[y] = y;
            if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
            }
         }
         else
         mLongBk[y] = EMPTY_VALUE;         
     
         if(bk[y] < 0)
         {
            if(bk[y-1] >= 0) 
            {
            mShortBk[y] = mhi;
            failsig[y] = 0;
            dir[y] =-1;
            LastShortBar[y] = y;
            if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
            }
         }         

        
         
         if(mhi < mUp[y] && mhi1 > mUp[y-1] && mUp[y] != mUp[y-1] && bk[y-1] > 0)
         { 
         mShortEx[y] = mhi;
         bk[y] = 0; 
         fail[y] = 0;
         dir[y] =-1;
         LastShortBar[y] = y;
         ex[y] =-1;
         if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
         } 
         else
         mShortEx[y] = EMPTY_VALUE;
      
     
               
         if(mlo > mDn[y] && mlo1 < mDn[y-1] && mDn[y] != mDn[y-1] && bk[y-1] < 0) //ex[y] =-1;
         {
         mLongEx[y] = mlo;
         bk[y] = 0; 
         fail[y] = 0;
         dir[y] = 1;
         LastLongBar[y] = y;
         ex[y] = 1;
         if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
         }
         else
         mLongEx[y] = EMPTY_VALUE;
      
       if(mlo < mDn[y]) fail[y] = 1;   
         
         if(mlo > mDn[y] && fail[y] > 0 && failsig[y] <= 0 ) 
         {
         mLongF[y] = mlo; 
         failsig[y] = 1; 
         bk[y]= 0;
         dir[y] = 1;
         LastLongBar[y] = y;
         if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
         } 
         else mLongF[y] = EMPTY_VALUE;   
              
         
      if(mhi > mUp[y]) fail[y] =-1;  
         
         if(mhi < mUp[y] && fail[y] < 0 && failsig[y] >= 0) 
         {
         mShortF[y] = mhi; 
         failsig[y] =-1; 
         bk[y]= 0;
         dir[y] =-1;
         LastShortBar[y] = y;
         if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
         } 
         else mShortF[y] = EMPTY_VALUE;
         
         
         if(ReversalSignals > 0)
         {
         a_name = "Revers" + setup + "_" + TimeToStr(Time[mBars-y-1]);
         ObjectDelete(a_name);  
          
         if(mlo > mDn[y] && mhi < mUp[y])
         { 
           
            
            if(dir[y] < 0)
            {
			      for(i = 0; i < PivotLength; i++)
			      {
                  if(mlo > hhArr[i] && mlo1 < hhArr[i]) 
				      {
					   bool allowBuy = false;
                  for(int j = LastShortBar[y] - y; j <= 0; j++) 
                     if(iLow(NULL,TimeFrame,mBars-y-1-j) < hhArr[i]) {allowBuy = true; break; } 
					
					      if(allowBuy)
					      { 
					      drawArrow("Revers",Time[mBars-y-1],NormalizeDouble(mlo,Digits),"Wingdings 3",11,"\xC7",Green);
                     if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
                     }
                  }	
               }	
			   }			
			
			   if(dir[y] > 0)
            {
			      for(i = 0; i < PivotLength; i++)
			      {
                  if(mhi < hhArr[i] && mhi1 > hhArr[i]) 
				      {
					   bool allowSell = false;
                  for(j = LastLongBar[y] - y; j <= 0; j++) 
                     if(iHigh(NULL,TimeFrame,mBars-y-1-j) > hhArr[i]) {allowSell = true; break; } 
					
					      if(allowSell)
					      {
					      drawArrow("Revers",Time[mBars-y-1],NormalizeDouble(mhi + 3*diff,Digits),"Wingdings 3",11,"\xC8",Red);
                     if(WarningMode>0 && y==mBars-1) PlaySound("alert2.wav"); 
                     }
                  }	
               }	
			   }			
			}
      }
   }
      
      
      
      
      if(TimeFrame == Period()) 
      {
      UpPivot[mBars-y-1] = mUp[y];
      LoPivot[mBars-y-1] = mDn[y];
         
         if(SignalMode > 0)
         {
         LongF[mBars-y-1]  = mLongF[y];
         ShortF[mBars-y-1] = mShortF[y];
         LongBk[mBars-y-1]   = mLongBk[y];
         ShortBk[mBars-y-1]  = mShortBk[y];
         LongEx[mBars-y-1]   = mLongEx[y];
         ShortEx[mBars-y-1]  = mShortEx[y];
         
         }
      
         
      
      //sig[mBars-y-1] = trend[y];  //Buffer for EAs
      //Print("sig=",sig[mBars-y-1]);
      }
      
   }
   mcnt_bars = mBars-1;
   
   if(TimeFrame > Period())
   { 
      if(cnt_bars>0) cnt_bars--;
      limit = Bars-cnt_bars+TimeFrame/Period()-1;
      
      for(shift=0,y=0;shift<limit;shift++)
      {
      if (Time[shift] < iTime(NULL,TimeFrame,y)) y++; 
      UpPivot[shift] = mUp[mBars-y-1];
      LoPivot[shift] = mDn[mBars-y-1];
         
         if(SignalMode > 0)
         {      
         LongF[shift] = mLongF[mBars-y-1];
         ShortF[shift] = mShortF[mBars-y-1];
         }
         
         
      //sig[shift] = trend[mBars-y-1];  //Buffer for EAs
      }
   }
//----------   
   if(SignalMode>0 && AlertMode > 0)
   { 
   bool bkbuy = mLongBk[mBars - 3] == EMPTY_VALUE && mLongBk[mBars - 2] != EMPTY_VALUE;
   BoxAlert(bkbuy,"DB_Strategy: Long Breakout, Open: "+DoubleToStr(Open[0],Digits));  
   
   bool bksell = mShortBk[mBars - 3] == EMPTY_VALUE && mShortBk[mBars - 2] != EMPTY_VALUE;
   BoxAlert(bksell,"DB_Strategy: Short Breakout, Open: "+DoubleToStr(Open[0],Digits)); 
   
   bool exbuy = mLongEx[mBars - 3] == EMPTY_VALUE && mLongEx[mBars - 2] != EMPTY_VALUE;
   BoxAlert(exbuy,"DB_Strategy: Long Exhaustion, Open: "+DoubleToStr(Open[0],Digits));  
   
   bool exsell = mShortEx[mBars - 3] == EMPTY_VALUE && mShortEx[mBars - 2] != EMPTY_VALUE;
   BoxAlert(exsell,"DB_Strategy: Short Exhaustion, Open: "+DoubleToStr(Open[0],Digits)); 
   
   bool fbuy = mLongF[mBars - 3] == EMPTY_VALUE && mLongF[mBars - 2] != EMPTY_VALUE;
   BoxAlert(fbuy,"DB_Strategy: Long Failure, Open: "+DoubleToStr(Open[0],Digits));  
   
   bool fsell = mShortF[mBars - 3] == EMPTY_VALUE && mShortF[mBars - 2] != EMPTY_VALUE;
   BoxAlert(fsell,"DB_Strategy: Short Failure, Open: "+DoubleToStr(Open[0],Digits)); 
      
      if(ReversalSignals > 0)
      {
      bool rbuy  = ObjectGet("Revers" + setup + "_" + TimeToStr(Time[1]),OBJPROP_PRICE1) == mlo1;
      BoxAlert(rbuy,"DB_Strategy: Possible Long Pivots Reversal, Open: "+DoubleToStr(Open[0],Digits)); 
      
      bool rsell  = ObjectGet("Revers" + setup + "_" + TimeToStr(Time[1]),OBJPROP_PRICE1) == mhi1 + 3*diff;
      BoxAlert(rsell,"DB_Strategy: Possible Short Pivots Reversal, Open: "+DoubleToStr(Open[0],Digits)); 
      }
      
      if(PivotSignals > 0)
      {
      bool pbuy  = ObjectGet("Pivots" + setup + "_" + TimeToStr(Time[1]),OBJPROP_PRICE1) == mlo1;
      BoxAlert(pbuy,"DB_Strategy: Possible Long Reversal, Open: "+DoubleToStr(Open[0],Digits)); 
      
      bool psell  = ObjectGet("Pivots" + setup + "_" + TimeToStr(Time[1]),OBJPROP_PRICE1) == mhi1 + 3*diff;
      BoxAlert(psell,"DB_Strategy: Possible Short Reversal, Open: "+DoubleToStr(Open[0],Digits)); 
      }
   }




//---- 
   return(0);
}

// MA_Method=0: SMA - Simple Moving Average
double SMA(double array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar-i];
   
   return(Sum/per);
}                
// MA_Method=1: EMA - Exponential Moving Average
double EMA(double price,double array[],int per,int bar)
{
   if(bar == 2) double ema = price;
   else 
   if(bar > 2) ema = array[bar-1] + 2.0/(1+per)*(price - array[bar-1]); 
   
   return(ema);
}
// MA_Method=2: Wilder - Wilder Exponential Moving Average
double Wilder(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double wilder = SMA(array1,per,bar);
   else 
   if(bar > per) wilder = array2[bar-1] + (array1[bar] - array2[bar-1])/per; 
   
   return(wilder);
}
// MA_Method=3: LWMA - Linear Weighted Moving Average 
double LWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar-i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
} 
// MA_Method=4: SineWMA - Sine Weighted Moving Average
double SineWMA(double array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
  
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(pi*(i+1)/(per+1));
      Sum += array[bar-i]*MathSin(pi*(i+1)/(per+1)); 
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}
// MA_Method=5: TriMA - Triangular Moving Average
double TriMA(double array[],int per,int bar)
{
   double sma;
   int len = MathCeil((per+1)*0.5);
   
   double sum=0;
   for(int i = 0;i < len;i++) 
   {
   sma = SMA(array,len,bar-i);
   sum += sma;
   } 
   double trima = sum/len;
   
   return(trima);
}
// MA_Method=6: LSMA - Least Square Moving Average (or EPMA, Linear Regression Line)
double LSMA(double array[],int per,int bar)
{   
   double Sum=0;
   for(int i=per; i>=1; i--) Sum += (i-(per+1)/3.0)*array[bar-per+i];
   double lsma = Sum*6/(per*(per+1));
   return(lsma);
}
// MA_Method=7: SMMA - Smoothed Moving Average
double SMMA(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double smma = SMA(array1,per,bar);
   else 
   if(bar > per)
   {
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array1[bar-i-1];
   smma = (Sum - array2[bar-1] + array1[bar])/per;
   }
   return(smma);
}                
// MA_Method=8: HMA - Hull Moving Average by Alan Hull
double HMA(double array[],int per,int bar)
{
   double tmp[];
   int len =  MathSqrt(per);
   ArrayResize(tmp,len);
   
   if(bar == per) double hma = array[bar]; 
   else
   if(bar > per)
   {
   for(int i = 0; i < len;i++) tmp[len-i-1] = 2*LWMA(array,per/2,bar-i) - LWMA(array,per,bar-i);  
   hma = LWMA(tmp,len,len-1); 
   }  

   return(hma);
}
// MA_Method=9: ZeroLagEMA - Zero-Lag Exponential Moving Average
double ZeroLagEMA(double array1[],double array2[],int per,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   
   if(bar == lag) double zema = array1[bar];
   else 
   if(bar > lag) zema = alfa*(2*array1[bar] - array1[bar-lag]) + (1-alfa)*array2[bar-1];
   
   return(zema);
}
// MA_Method=10: DEMA - Double Exponential Moving Average by Patrick Mulloy
double DEMA(int num,double price,int per,double v,int bar)
{
   if(bar == 2) {double dema = price; tmp[bar][num] = dema; tmp[bar][num+1] = dema;}
   else 
   if(bar > 2) 
   {
   tmp[bar][num] = tmp[bar-1][num] + 2.0/(1+per)*(price - tmp[bar-1][num]); 
   tmp[bar][num+1] = tmp[bar-1][num+1] + 2.0/(1+per)*(tmp[bar][num] - tmp[bar-1][num+1]); 
   dema = (1+v)*tmp[bar][num] - v*tmp[bar][num+1];
   }
   return(dema);
}
// MA_Method=11: T3 by T.Tillson
double T3(int num,double price,int per,double v,int bar)
{
   if(bar == 2) 
   {
   double T3 = price; 
   for(int k=0;k<=5;k++) tmp[bar][k] = T3;
   }
   else 
   if(bar > 2) 
   {
   double dema1 = DEMA(num,price,per,v,bar); 
   double dema2 = DEMA(num+2,dema1,per,v,bar); 
   T3 = DEMA(num+4,dema2,per,v,bar);
   }
   return(T3);
}
// MA_Method=12: ITrend - Instantaneous Trendline by J.Ehlers
double ITrend(double price[],double array[],int per,int bar)
{
   double alfa = 2.0/(per+1);
   if(bar > 7)
   double it = (alfa - 0.25*alfa*alfa)*price[bar]+ 0.5*alfa*alfa*price[bar-1]-(alfa - 0.75*alfa*alfa)*price[bar-2]+
   2*(1-alfa)*array[bar-1] - (1-alfa)*(1-alfa)*array[bar-2];
   else
   it = (price[bar] + 2*price[bar-1]+ price[bar-2])/4;
   
   return(it);
}
// MA_Method=13: Median - Moving Median
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar-i];
   ArraySort(array);
   
   int num = MathRound((per-1)/2); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}
// MA_Method=14: GeoMean - Geometric Mean
double GeoMean(double price[],int per,int bar)
{
   double gmean = MathPow(price[bar],1.0/per); 
   for(int i = 1; i < per;i++) gmean *= MathPow(price[bar-i],1.0/per); 
   
   return(gmean);
}
// MA_Method=15: REMA - Regularized EMA by Chris Satchwell 
double REMA(double price,double array[],int per,double lambda,int bar)
{
   double alpha =  2.0/(per + 1);
   if(bar <= 3) double rema = price;
   else 
   if(bar > 3) 
   rema = (array[bar-1]*(1+2*lambda) + alpha*(price - array[bar-1]) - lambda*array[bar-2])/(1+lambda); 
   
   return(rema);
}
// MA_Method=16: ILRS - Integral of Linear Regression Slope 
double ILRS(double price[],int per,int bar)
{
   double sum = per*(per-1)*0.5;
   double sum2 = (per-1)*per*(2*per-1)/6.0;
     
   double sum1 = 0;
   double sumy = 0;
      for(int i=0;i<per;i++)
      { 
      sum1 += i*price[bar-i];
      sumy += price[bar-i];
      }
   double num1 = per*sum1 - sum*sumy;
   double num2 = sum*sum - per*sum2;
   
   if(num2 != 0) double slope = num1/num2; else slope = 0; 
   double ilrs = slope + SMA(price,per,bar);
   
   return(ilrs);
}
// MA_Method=17: IE/2 - Combination of LSMA and ILRS 
double IE2(double price[],int per,int bar)
{
   double ie = 0.5*(ILRS(price,per,bar) + LSMA(price,per,bar));
      
   return(ie); 
}
 
// MA_Method=18: TriMAgen - Triangular Moving Average Generalized by J.Ehlers
double TriMA_gen(double array[],int per,int bar)
{
   int len1 = MathFloor((per+1)*0.5);
   int len2 = MathCeil((per+1)*0.5);
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMA(array,len1,bar-i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

// MA_Method=19: VWMA - Volume Weighted Moving Average 
double VWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= Volume[mBars-bar-1+i];
      Sum += array[bar-i]*Volume[mBars-bar-1+i];
      }
   if(Weight>0) double vwma = Sum/Weight;
   else vwma = 0; 
   return(vwma);
} 

// MA_Method=20: JSmooth - Smoothing by Mark Jurik
double JSmooth(int num,double price,int per,double pow,int bar)
{
   double beta = 0.45*(per-1)/(0.45*(per-1)+2);
	double alpha = MathPow(beta,pow);
	if(bar == 2) {tmp[bar][num+4] = price; tmp[bar][num+0] = price; tmp[bar][num+2] = price;}
	else 
   if(bar > 2) 
   {
	tmp[bar][num+0] = (1-alpha)*price + alpha*tmp[bar-1][num+0];
	tmp[bar][num+1] = (price - tmp[bar][num+0])*(1-beta) + beta*tmp[bar-1][num+1];
	tmp[bar][num+2] = tmp[bar][num+0] + tmp[bar][num+1];
	tmp[bar][num+3] = (tmp[bar][num+2] - tmp[bar-1][num+4])*MathPow((1-alpha),2) + MathPow(alpha,2)*tmp[bar-1][num+3];
	tmp[bar][num+4] = tmp[bar-1][num+4] + tmp[bar][num+3]; 
   }
   return(tmp[bar][num+4]);
}

void drawArrow(string prefix,int time,double price,string font,int size,string acode,color clr)
{
   string c_name = prefix + setup + "_" + TimeToStr(time);  
   ObjectCreate(c_name, OBJ_TEXT, 0,time, price);
   ObjectSetText(c_name,acode,size,font,clr); 
}

bool ObjectDel()
{
   int _GetLastError = 0;
     
   while(ObjFind(setup,0,6) > 0)
   {
      int obtotal = ObjectsTotal();
      
      for (int i = 0; i < obtotal;i++)
      {
         if (StringFind(ObjectName(i),setup,0) >= 0)
         {
            if (!ObjectDelete(ObjectName(i)))
            {
            _GetLastError = GetLastError();
            Print( "ObjectDelete( \"",ObjectName(i),"\" ) - Error #", _GetLastError );
            }
         }
      }
   }      
}
//-----
int ObjFind(string name,int start, int num)
{
   int cnt = 0;
   
   for (int i = 0; i < ObjectsTotal();i++)
      if (StringFind(ObjectName(i),name,start) == num) cnt+=1;
   
   return(cnt);
} 

bool BoxAlert(bool cond,string text)   
{      
   string mess = " "+Symbol()+" "+TF+":" + text;
   
   if (cond && mess != prevmess)
	{
	Alert (mess);
	prevmess = mess; 
	return(true);
	} 
}	


